package wav;

import header.WavFileHeader;
import header.WaveFormatEx;

import java.io.IOException;
import java.io.RandomAccessFile;

/**
 * Erzeugung von Wave-Dateien mit einem Sinuston  
 * @author Michael Steppat
 *
 */

public class SinusWav {
	/**
	 * Referenz fr den Zugriff auf die zu schreibende Datei
	 */
	private RandomAccessFile file;		
	/**
	 * Das WaveFormatEx-Objekt enthlt die Formatdaten der Audiodatei
	 */
	private WaveFormatEx waveFormatEx;	
	
	/**
	 * 
	 * Erzeugt eine leere Audiodatei mit einem Header.
	 * 
	 * @param fileName Name der zu erzeugenden Datei.
	 * @throws IOException Wird ausgelst, wenn auf das Verzeichnis nicht geschrieben werden kann
	 */
	public void createFile(String fileName) throws IOException {
		
		file = new RandomAccessFile(fileName, "rw");
		file.setLength(0);
		int i;		
		for(i = 0; i < 44; i++)
			file.writeByte(0);
		waveFormatEx = new WaveFormatEx();
		waveFormatEx.wFormatTag = 1;
		waveFormatEx.nChannels = 1;
		waveFormatEx.nSamplesPerSec = 44100;
		waveFormatEx.nAvgBytesPerSec = 88200;
		waveFormatEx.nBlockAlign = 2;
		waveFormatEx.wBitsPerSample = 16;
		waveFormatEx.cbSize = 0;			
	}
	
	/**
	 * Erzeugt eine Sinusschwingung
	 * 
	 * @param amplitude Amplitudenwert 0...32767
	 * @param frequency Frequenz in Hz
	 * @param phase Phasenlage in 
	 * @param length Anzahl der zu erzeugenden Samples
	 * @throws IOException Wird ausgelst, wenn ein Fehler beim Schreiben auftritt
	 */
	public void createAudioData(double amplitude, double frequency, double phase, int length) throws IOException {		
		int i;
		phase = phase * Math.PI / 180;
		short sample;
		for(i = 0; i < length; i++) {
			sample = (short)(amplitude * Math.sin(phase + 2 * Math.PI * frequency * i / 44100));
			file.writeByte(sample);
			file.writeByte(sample >> 8);
		}
	}
	/**
	 * Schreibt die Headerdaten in die Datei 
	 * @throws IOException Wird bei Schreibfehlern ausgelst
	 */
	public void writeHeader() throws IOException
	{
		WavFileHeader wavFileHeader = new WavFileHeader();
		wavFileHeader.writeHeader(file, waveFormatEx);		
	}
	/**
	 * Hauptfunktion 
	 * @param args Pfad und Name der zu erzeugenden Datei, Amplitudenwert 0...32767, Frequenz in Hz, Phasenlage in , Anzahl der zu erzeugenden Samples 
	 */
	public static void main(String[] args) {		
		double amplitude = Double.valueOf(args[1]); 
		double frequency = Double.valueOf(args[2]); 
		double phase = Double.valueOf(args[3]); 
		int length = Integer.valueOf(args[4]);
		
		SinusWav sinus = new SinusWav();			
		try {		
			sinus.createFile(args[0]);
			sinus.createAudioData(amplitude, frequency, phase, length);
			sinus.writeHeader();			
		} catch (IOException e) {			
			e.printStackTrace();
		}
		System.out.println("Wavedatei mit Sinusschwingung erzeugt");
	}
}
