package visualisation;
/**
 * Hilfsklasse mit Methoden zur Durchfhrung der FFT und IFFT (Rcktransformation).
 * @author Michael Steppat
 *
 */

public class FourierTransform {
	/**
	 * Aufruf der FFT. Die Transformation ist eine in-place-transformation. Daher werden die Werte in den Eingabepuffern xr, xi mit den Ausgabewerten berschrieben. 
	 * @param xr Samples der Zeitfunktion (Eingabe), Reelle Spektralwerte (Ausgabe)
	 * @param xi Bei der Eingabe sind alle Werte 0, Imaginre Spektralwerte (Ausgabe)
	 * @param n Anzahl der Sttzstellen (muss eine Zweierpotenz sein)
	 */
	
	public void fft(float xr[], float xi[], int n)	{
		inversion(xr, xi, n);
		transform(xr, xi, n, 0);
		for(int i = 0; i < n; i++) {
			xr[i] /= (n / 2);
			xi[i] /= (n / 2);
		}
	}
	/**
	 * Aufruf der inversen FFT (Rcktransformation). Die Transformation ist eine in-place-transformation. 
	 * Daher werden die Werte in den Eingabepuffern xr, xi mit den Ausgabewerten berschrieben.
	 * @param xr Reelle Spektralwerte (Eingabe), Zeitfunktion (Ausgabe)
	 * @param xi Imaginre Spektralwerte (Eingabe), Nullfolge (Ausgabe) 
	 * @param n Anzahl der Sttzstellen (muss eine Zweierpotenz sein)
	 */

	public void ifft(float xr[], float xi[], int n)	{
		inversion(xr, xi, n);
		transform(xr, xi, n, 1);
		for(int i = 0; i < n; i++) {
			xr[i] /= 2;
			xi[i] /= 2;
		}
	}
	/**
	 * Durchfhrung der Transformation
	 * @param xr Reelle Ein- und Ausgabewerte
	 * @param xi Imaginre Ein- und Ausgabewerte 
	 * @param n Anzahl der Sttzstellen (muss eine Zweierpotenz sein)
	 * @param typ 0 fr FFT, 1 fr IFFT
	 */
	private void transform(float xr[], float xi[], int n, int typ)	{
		int i, j, k, l, m;
		int le;
		float hr,hi;
		float wr, wi, whr, whi;

		for(i = n, m = 0; i > 1; m++, i /= 2); //ld(n) bestimmen
		for(i = 0; i < m; i++) {
			le = 1 << (i + 1);
			whr = 1;
			whi = 0;
			if(typ == 0) {
				wr = (float)Math.cos(Math.PI / le * 2);
				wi = (float)-Math.sin(Math.PI / le * 2);
			}
			else {
				wr = (float)Math.cos(Math.PI / le * 2);
				wi = (float)Math.sin(Math.PI / le * 2);
			}

			for(j = 0; j < le / 2; j++) {
				for(k = j; k < n; k += le) {
					l = k + le / 2;
					hr = xr[l] * whr - xi[l] * whi; 
					hi = xi[l] * whr + xr[l] * whi;
					xr[l] = xr[k] - hr; 
					xi[l] = xi[k] - hi;
					xr[k] = xr[k] + hr;
					xi[k] = xi[k] + hi;
				}
				hr = whr * wr - whi * wi;
				hi = whi * wr + whr * wi;
				whr = hr;
				whi = hi;
			}
		}
	}
	/**
	 * Sortierung der Eingabewerte
	 * @param xr Reelle Ein- und Ausgabewerte
	 * @param xi Imaginre Ein- und Ausgabewerte 
	 * @param n Anzahl der Sttzstellen (muss eine Zweierpotenz sein)	
	 */
	private void inversion(float xr[], float xi[], int n) {
		int		i, j, k;
		float	hr, hi;

		for(i = j = 1, k = n/2; i < n; i++, j += k, k = n / 2) {
			if(j > i) {
				hr = xr[i - 1];
				hi = xi[i - 1];
				xr[i - 1] = xr[j - 1];
				xi[i - 1] = xi[j - 1];
				xr[j - 1] = hr;
				xi[j - 1] = hi;
			}
			for(; k < j; j -= k, k /= 2);
		}
	}      

}
